SystemOrganization addCategory: #'SqueakNxt-Compiler-Tests'!
SystemOrganization addCategory: #'SqueakNxt-Compiler'!

TestCase subclass: #NbcTranslatorTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler-Tests'!

RBProgramNodeVisitor subclass: #NbcTranslator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler'!

Object subclass: #Nbc
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler'!

!Nbc commentStamp: 'Richo 12/25/2010 01:12' prior: 0!
A NbcExe is a very simple API for accessing the nbc compiler using FFI on Windows

Examples: 
1) The simplest example would be compiling into a file called "output" and reporting errors to a file called "errors":

	NbcExe new
		fileName: 'example_program.nbc';
		outFile: 'output';
		errorsFile: 'errors';
		execute.
		
2) To download a program into the Lego Nxt using the usb:

	NbcExe new
		fileName: 'example_program.nbc';
		portName: 'usb';
		downloadProgram;
		execute.
		
3) To use the bluetooth on COM6 instead:
	
	NbcExe new
		fileName: 'example_program.nbc';
		useBluetooth;
		portName: 'COM6';
		downloadProgram;
		execute.

Instance Variables
	parameters:		<WriteStream>

parameters
	- Used to build the parameters string to be sent to the nbc program. See "nbc -help" for more information.
!

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
beQuiet
	parameters space; nextPutAll: '-q'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
decompileProgram
	parameters space; nextPutAll: '-x'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
defineMacro: aByteString 
	parameters space; nextPutAll: '-D='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontCompile
	parameters space; nextPutAll: '-b'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:45'!
dontIncludeSystemFile
	parameters space; nextPutAll: '-n'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
downloadProgram
	parameters space; nextPutAll: '-d'! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
errorsFile: aByteString 
	parameters space; nextPutAll: '-E='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'executing' stamp: 'Richo 12/25/2010 01:02'!
execute
	"Executes the command using FFI. Windows only for now"
	| result shell |
	shell := Smalltalk at: #Win32Shell ifAbsent: [^self error: 'FFI doesn''t seem to be installed'].
	(result := (shell new 
		shellExecute:  nil
		lpOperation:  'open'
		lpFile:  'nbc.exe'
		lpParameters:  parameters contents
		lpDirectory:  nil
		nShowCmd: 1) <= 32) ifTrue: [self error: 'system error, code:', result]! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:46'!
fileName: aByteString 
	parameters space; nextPutAll: aByteString! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
includePath: aByteString 
	parameters space; nextPutAll: '-I='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'initialize' stamp: 'Richo 12/25/2010 00:44'!
initialize
	super initialize.
	parameters := WriteStream on: ''! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
outFile: aByteString 
	parameters space; nextPutAll: '-O='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:47'!
portName: aByteString 
	parameters space; nextPutAll: '-S='; nextPutAll: aByteString ! !

!Nbc methodsFor: 'accessing' stamp: 'Richo 12/25/2010 00:48'!
useBluetooth
	parameters space; nextPutAll: '-BT'! !

Object subclass: #NbcCompiler
	instanceVariableNames: 'programName source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler'!

!NbcCompiler commentStamp: 'Richo 12/27/2010 18:33' prior: 0!
A NbcCompiler is responsible for verifying/compiling programs as well as downloading them to the LegoNxt.

===============================
	"Code example"
	
	| program |
	
	"This nbc program simply plays five tones on the nxt"
	program := '
		thread main
			PlayTone(TONE_C5, 500)
			wait 500
			PlayTone(TONE_D5, 500)
			wait 500
			PlayTone(TONE_E5, 500)
			wait 500
			PlayTone(TONE_D5, 500)
			wait 500
			PlayTone(TONE_C5, 1000)
			wait 1000
		endt'.

	"The code below will verify the program and return a collection of errors (if any)"
	NbcCompiler new
		source: program;
		verify.
	
	"The code below will download the program to the nxt using the usb cable. the resulting program will be called 'PlayingMusic'"
	NbcCompiler new
		programName: 'PlayingMusic';
		source: program;
		downloadUsingUsb.
===============================

Instance Variables
	programName:		<String>
	source:		<String>

programName
	- A name for the program. If not specified the program name will be the NbcCompiler instance's hash (not a nice name for a program, I must say).

source
	- The nbc source code of the program.
!

!NbcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:31'!
compileInFileNamed: outFile
	"Compiles the program given as a parameter and returns a collection of NbcErrors. If the collection is empty then the program is ok"
	^ self execute: (Nbc new outFile: outFile)! !

!NbcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:10'!
downloadUsingBluetooth: portName 
	"Compiles the program and downloads it using the bluetooth on the specified port name (COMn). Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: portName; useBluetooth; downloadProgram)! !

!NbcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 18:09'!
downloadUsingUsb
	"Compiles the program and downloads it using the usb cable. Returns a collection of NbcErrors"
	^ self execute: (Nbc new portName: 'usb'; downloadProgram)! !

!NbcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:14'!
errorsFileName
	"Temporary file name to store errors during compilation"
	^ self programName , '.err'! !

!NbcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:18'!
execute: nbc 
	"Create the temporary files to store the source code and the errors.
	Execute nbc.
	Wait until nbc finishes and parse errors.
	Delete temporary files.
	Return errors"
	source isNil ifTrue: [^ self error: 'Source cannot be nil'].
	FileStream forceNewFileNamed: self programFileName do: [:file | file nextPutAll: source]; 
				forceNewFileNamed: self errorsFileName do: [:file | "Do nothing. Nbc will write this file"].
	[nbc fileName: self programFileName;
		 errorsFile: self errorsFileName;
		 execute.
	^ self parseErrors]
		ensure: [FileDirectory default deleteFileNamed: self programFileName;
				 deleteFileNamed: self errorsFileName]! !

!NbcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:12'!
parse: errorsFile
"Returns a collection with all the errors founded by the nbc compiler. The errorsFile format is similar to this:

# Error: Unknown or invalid statement
File ""D:\RICHI\Squeak images\Nxt\simpleprogram.nbc"" ; line 7
#   result sbyt
#----------------------------------------------------------
1 error during compilation
"
| errors |
errors := OrderedCollection new.
[errorsFile peek = $#] 
	whileTrue: [ | errorDescription errorPosition errorSource |
		errorDescription := errorsFile nextLine allButFirst: 2.
		errorPosition := (errorsFile nextLine subStrings: {$;}) last withoutLeadingBlanks.
		errorSource := errorsFile nextLine allButFirst: 4.
		errorsFile nextLine.
		errors add: (NbcError description: errorDescription position: errorPosition source: errorSource).
		].
^errors asArray! !

!NbcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:38'!
parseErrors
	"Wait until the errors file is fully written by nbc, then read it and return the errors.
	IMPORTANT: Be aware that this code relies on the fact that FileStream>>#fileNamed: returns nil when the file is being modified by another process. If this somehow changes in the future, this code will inevitably break."
	| errorsFile |
	errorsFile := nil.
	[errorsFile isNil]
		whileTrue: [500 milliSeconds asDelay wait.
			errorsFile := FileStream fileNamed: self errorsFileName].
	^ [self parse: errorsFile]
		ensure: [errorsFile close].
! !

!NbcCompiler methodsFor: 'private' stamp: 'Richo 12/27/2010 18:14'!
programFileName
	"Temporary file name to store the source code"
	^ self programName , '.nbc'! !

!NbcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 17:07'!
programName
	^ programName ifNil: [self hash asString]! !

!NbcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:47'!
programName: aString
	programName := aString! !

!NbcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 17:07'!
source
	^ source! !

!NbcCompiler methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:48'!
source: aString
	source := aString.! !

!NbcCompiler methodsFor: 'compiling' stamp: 'Richo 12/27/2010 17:48'!
verify
	"Compiles the program given as a parameter and returns a collection of NbcErrors. If the collection is empty then the program is ok"
	^ self execute: Nbc new! !

Object subclass: #NbcError
	instanceVariableNames: 'description position source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakNxt-Compiler'!

!NbcError class methodsFor: 'instance creation' stamp: 'Richo 12/25/2010 16:02'!
description: description position: position source: source
^ (self basicNew setDescription: description position: position source: source) initialize ! !

!NbcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
description
	^ description ! !

!NbcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
position
	^ position ! !

!NbcError methodsFor: 'printing' stamp: 'Richo 12/27/2010 18:21'!
printOn: aStream
aStream nextPutAll: position asString;
		 nextPutAll: ' : ';
		 nextPutAll: description asString 
! !

!NbcError methodsFor: 'private' stamp: 'Richo 12/25/2010 16:02'!
setDescription: aByteString position: aByteString2 source: aByteString3 
	description := aByteString.
	position := aByteString2.
	source := aByteString3! !

!NbcError methodsFor: 'accessing' stamp: 'Richo 12/27/2010 16:01'!
source
	^ source ! !
